
BEGIN TRANSACTION
GO
PRINT N'Refreshing [dbo].[v_Security_ErrorLog]'
GO
EXEC sp_refreshview N'[dbo].[v_Security_ErrorLog]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO
PRINT N'Refreshing [dbo].[v_Security_DataLog]'
GO
EXEC sp_refreshview N'[dbo].[v_Security_DataLog]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO
PRINT N'Altering [dbo].[t_Project_Person]'
GO
ALTER TABLE [dbo].[t_Project_Person] ADD
[Email] [nvarchar] (255) COLLATE Latin1_General_CI_AS NULL
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Altering [dbo].[t_Project_OrganizationUnit]'
GO
ALTER TABLE [dbo].[t_Project_OrganizationUnit] ADD
[AbsenceHandlerRef] [uniqueidentifier] NULL
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Altering [dbo].[vc_DataSource_Person]'
GO


ALTER VIEW [dbo].[vc_DataSource_Person]

As

--- Standardized reporting-view for t_Project_Person records ---

Select

	--- Standard columns
	pers.OrganizationUnitRef, pers.RevisionRef, 
	pers.[Login], pers.PersonID, pers.Firstname, pers.Lastname, 
	pers.CalendarRef, pers.HiredFrom, pers.HiredTo, pers.UserRef, pers.PrimKey, pers.[TimeStamp],

	--- Combined columns
	isnull(pers.Firstname,'') + ' ' + isnull(pers.Lastname,'') As PersonName,
	isnull(pers.PersonID,'') + ' - ' + isnull(pers.Firstname,'') + ' ' + isnull(pers.Lastname,'') As PersonIDAndName,

	--- Related columns
	ou.OrganizationUnitCode, ou.Title OrganizationUnitTitle, ou.[Description] OrganizationUnitDescription,
	cal.Title CalendarTitle, cal.[Description] CalendarDescription,

	ou.ParentRef OrgUnitParentRef, oup.OrganizationUnitCode ParentOrganizationUnitCode, oup.Title ParentOrganizationUnitTitle, oup.[Description] ParentOrganizationUnitDescription

From 
	t_Project_Revision rev with(nolock) 
	Inner Join t_Project_Person pers with(nolock) On pers.RevisionRef = rev.PrimKey 
	Left Outer Join t_Project_OrganizationUnit ou with(nolock) on pers.OrganizationUnitRef = ou.PrimKey
	Left Outer Join t_Project_OrganizationUnit oup with(nolock) on ou.ParentRef = oup.PrimKey
	Inner Join t_Project_Calendar cal with(nolock) On cal.PrimKey = pers.CalendarRef

Where 
	rev.CurrentRevision = 1


GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[v_MPP_ProjectPositionValue]'
GO
EXEC sp_refreshview N'[dbo].[v_MPP_ProjectPositionValue]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Altering [dbo].[t_Project_ProjectRevision]'
GO
ALTER TABLE [dbo].[t_Project_ProjectRevision] ADD
[ImportID] [varchar] (100) COLLATE Latin1_General_CI_AS NULL
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating [dbo].[t_Project_ProjectType]'
GO
CREATE TABLE [dbo].[t_Project_ProjectType]
(
[RevisionRef] [uniqueidentifier] NOT NULL,
[Title] [nvarchar] (100) COLLATE Latin1_General_CI_AS NOT NULL,
[Description] [nvarchar] (max) COLLATE Latin1_General_CI_AS NULL,
[PrimKey] [uniqueidentifier] NOT NULL CONSTRAINT [DF_t_Project_ProjectType_PrimKey] DEFAULT (newsequentialid()),
[TimeStamp] [timestamp] NOT NULL
)
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating primary key [PK_t_Project_ProjectType] on [dbo].[t_Project_ProjectType]'
GO
ALTER TABLE [dbo].[t_Project_ProjectType] ADD CONSTRAINT [PK_t_Project_ProjectType] PRIMARY KEY CLUSTERED  ([PrimKey])
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating [dbo].[v_Project_ProjectType]'
GO
CREATE VIEW [dbo].[v_Project_ProjectType]
AS
SELECT * FROM [t_Project_ProjectType] with (nolock)
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating [dbo].[t_Project_ProjectStatus]'
GO
CREATE TABLE [dbo].[t_Project_ProjectStatus]
(
[RevisionRef] [uniqueidentifier] NOT NULL,
[Title] [nvarchar] (100) COLLATE Latin1_General_CI_AS NOT NULL,
[Description] [nvarchar] (max) COLLATE Latin1_General_CI_AS NULL,
[PrimKey] [uniqueidentifier] NOT NULL CONSTRAINT [DF_t_Project_ProjectStatus_PrimKey] DEFAULT (newsequentialid()),
[TimeStamp] [timestamp] NOT NULL
)
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating primary key [PK_t_Project_ProjectStatus] on [dbo].[t_Project_ProjectStatus]'
GO
ALTER TABLE [dbo].[t_Project_ProjectStatus] ADD CONSTRAINT [PK_t_Project_ProjectStatus] PRIMARY KEY CLUSTERED  ([PrimKey])
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating [dbo].[v_Project_ProjectStatus]'
GO

CREATE VIEW [dbo].[v_Project_ProjectStatus]
AS
SELECT * FROM [t_Project_ProjectStatus] with (nolock)
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[v_Project_ProjectRevision]'
GO
EXEC sp_refreshview N'[dbo].[v_Project_ProjectRevision]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[vc_Security_MyRoles]'
GO
EXEC sp_refreshview N'[dbo].[vc_Security_MyRoles]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Altering [dbo].[t_Project_Project]'
GO
ALTER TABLE [dbo].[t_Project_Project] ADD
[ProjectTypeRef] [uniqueidentifier] NULL,
[ProjectStatusRef] [uniqueidentifier] NULL
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[v_Project_Project]'
GO
EXEC sp_refreshview N'[dbo].[v_Project_Project]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[v_Project_Person]'
GO
EXEC sp_refreshview N'[dbo].[v_Project_Person]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[v_Project_OrganizationUnit]'
GO
EXEC sp_refreshview N'[dbo].[v_Project_OrganizationUnit]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[v_MPP_ProjectPosition]'
GO
EXEC sp_refreshview N'[dbo].[v_MPP_ProjectPosition]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[v_MPP_OrganizationUnitPositionType]'
GO
EXEC sp_refreshview N'[dbo].[v_MPP_OrganizationUnitPositionType]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Altering [dbo].[vc_DataSource_ProjectPosition]'
GO

ALTER VIEW [dbo].[vc_DataSource_ProjectPosition]
AS

select 
	--- Standard columns (Position)
	pos.PositionID, pos.Title, pos.[Description], 
	pos.ProjectRevisionRef, pos.OrganizationUnitPositionTypeRef, pos.PersonRef, pos.CalendarRef, pos.PrimKey, pos.[TimeStamp],

	--- Related columns (Project)
	proj.ProjectCode, proj.Title ProjectTitle,  proj.[Description] ProjectDescription, 
	proj.ProjectTypeRef, proj.ProjectStatusRef, 
	
	--- Related columns (Project revision)
	projrev.Title As ProjectRevisionTitle, projrev.[Description] As ProjectRevisionDescription, 
	projrev.ProjectRef, 

	--- Related columns (Calendar)
	cal.Title CalendarTitle, cal.[Description] CalendarDescription, 

	--- Related column (Position type)
	pt.Title PosTypeTitle, pt.[Description] PosTypeDescription,

	--- Related colum (Organisation unit for POSITION)
	oupt.OrganizationUnitRef, oupt.PositionTypeRef,
	ou.OrganizationUnitCode, ou.Title OrganizationTitle, ou.[Description] OrganizationDescription,

	--- Related column (Person)
	pers.PersonID, pers.Firstname, pers.Lastname,
	isnull(pers.Firstname,'') + ' ' + isnull(pers.Lastname,'') PersonName,
	isnull(pers.PersonID,'') + ' - ' + isnull(pers.Firstname,'') + ' ' + isnull(pers.Lastname,'') PersonIDAndName,
	
	--- Related colum (Organisation unit for PERSON)
	pers.OrganizationUnitRef PersonOrganizationUnitRef,
	ox.OrganizationUnitCode PersonOrganizationUnitCode, ox.Title PersonOrganizationTitle, ox.[Description] PersonOrganizationDescription,
	
	--- Related column (Project type)
	projtype.Title ProjectTypeTitle, projtype.[Description] ProjectTypeDescription,

	--- Related column (Project status)
	projstat.Title ProjectStatusTitle, projstat.[Description] ProjectStatusDescription

from 
	dbo.v_Project_Revision rev with(nolock)
	inner join dbo.v_Project_ProjectRevision projrev with(nolock) on rev.PrimKey = projrev.RevisionRef
	inner join dbo.v_Project_Project proj with(nolock) on projrev.ProjectRef = proj.PrimKey and projrev.PrimKey = proj.LiveProjectRevisionRef
	inner join dbo.v_MPP_ProjectPosition pos with(nolock) on proj.LiveProjectRevisionRef = pos.ProjectRevisionRef
	left outer join dbo.v_Project_Person pers with(nolock) on pers.RevisionRef = rev.PrimKey and pers.PrimKey = pos.PersonRef
	left outer join dbo.v_Project_Calendar cal with(nolock) on cal.RevisionRef = rev.PrimKey and cal.PrimKey = pos.CalendarRef
	left outer join dbo.v_MPP_OrganizationUnitPositionType oupt with(nolock) on pos.OrganizationUnitPositionTypeRef = oupt.PrimKey
	left outer join dbo.v_MPP_PositionType pt with(nolock) on oupt.PositionTypeRef = pt.PrimKey 
	left outer join dbo.v_Project_OrganizationUnit ou with(nolock) on ou.PrimKey = oupt.OrganizationUnitRef
	left outer join dbo.v_Project_OrganizationUnit ox with(nolock) on ox.PrimKey = pers.OrganizationUnitRef
	left outer join dbo.v_Project_ProjectType projtype with(nolock) on projtype.PrimKey = proj.ProjectTypeRef
	left outer join dbo.v_Project_ProjectStatus projstat with(nolock) on projstat.PrimKey = proj.ProjectStatusRef

Where 
	rev.CurrentRevision = 1






GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Altering [dbo].[t_Project_PersonAvailability]'
GO
ALTER TABLE [dbo].[t_Project_PersonAvailability] ADD
[ImportID] [nvarchar] (100) COLLATE Latin1_General_CI_AS NULL
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[vc_MPP_CapacityAndNeed]'
GO
EXEC sp_refreshview N'[dbo].[vc_MPP_CapacityAndNeed]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[vc_DataSource_OrganizationUnit]'
GO
EXEC sp_refreshview N'[dbo].[vc_DataSource_OrganizationUnit]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Altering [dbo].[pc_DataSource_GetCustomFieldSQL]'
GO
ALTER PROCEDURE [dbo].[pc_DataSource_GetCustomFieldSQL](@TableName NVARCHAR(128), @Alias NVARCHAR(128)=NULL, @RefColumn NVARCHAR(128)='PrimKey')
AS
BEGIN
	DECLARE @allSelects nvarchar(max) Set @allSelects = ''
	DECLARE @allJoins nvarchar(max) Set @allJoins = ''
	DECLARE @errmsg nvarchar(max) Set @errmsg = ''
	DECLARE @outputTable table
	( 
		FieldList nvarchar(MAX), 
		LeftJoin nvarchar(MAX)
	)

	set nocount on 
	
	IF (@TableName Not In (
			't_MPP_PositionType',
			't_MPP_ProjectPosition',
			't_Project_ExpMhrs',
			't_Project_OrganizationUnit',
			't_Project_Person',
			't_Project_ProjectRevision')) 
		Set @errmsg = 'Custom fields for table [' + @TableName + '] not supported!'
		
	IF (@Alias IS NULL) Set @Alias = @TableName

	
	Declare @m_FieldTitle nvarchar(255)
	Declare @m_FieldType nvarchar(255)
	Declare @m_FieldRef uniqueidentifier		
	
	Declare @tbl_prefix varchar(255)
	Declare @tbl varchar(255)
	Declare @col_prefix varchar(255)
	Declare @col varchar(255)
	
	Declare @cf_count int 
	Declare @fieldname varchar(255)
	DECLARE @select nvarchar(max)
	DECLARE @join nvarchar(max)
	
	Declare @fieldtitlepattern varchar(50)  Set @fieldtitlepattern = '%[^a-z]%'

	
	IF (@TableName = 't_MPP_PositionType')
	BEGIN
		Declare my_cursor Cursor For
		--- Collect custom fields used for position types
		select f.title, f.[type], ptf.FieldRef
		from t_MPP_PositionTypeField ptf
		inner join t_Project_Revision r on ptf.RevisionRef = r.PrimKey
		left outer join t_Project_Field f on ptf.FieldRef = f.PrimKey
		where r.CurrentRevision = 1
		group by f.title, f.[type], ptf.FieldRef
		order by f.title, f.[type], ptf.FieldRef
	
		set @col_prefix = 'CF_' + @alias + '_'
		set @tbl_prefix = 't_'
		
		set @cf_count = 100
	
		Open my_cursor
		Fetch Next From my_cursor Into @m_FieldTitle, @m_FieldType, @m_FieldRef
		While @@FETCH_STATUS = 0
		Begin
			set @cf_count = @cf_count + 1
			While PatIndex(@fieldtitlepattern, @m_FieldTitle) > 0 Set @m_FieldTitle = Stuff(@m_FieldTitle, PatIndex(@fieldtitlepattern, @m_FieldTitle), 1, '')
	
			set @tbl = @tbl_prefix +  convert(varchar,@cf_count)
			set @col = @col_prefix + @m_FieldTitle + '_' + convert(varchar,@cf_count)

			IF (@m_FieldType = 'Date') set @select = @tbl + '.DateTimeValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Date') set @join = 'Left Outer Join v_MPP_PositionTypeValue ' + @tbl + ' On ' + @tbl + '.PositionTypeRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
	
			IF (@m_FieldType = 'Int') set @select = @tbl + '.IntValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Int') set @join = 'Left Outer Join v_MPP_PositionTypeValue ' + @tbl + ' On ' + @tbl + '.PositionTypeRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
	
			IF (@m_FieldType = 'Text') set @select = @tbl + '.TextValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Text') set @join = 'Left Outer Join v_MPP_PositionTypeValue ' + @tbl + ' On ' + @tbl + '.PositionTypeRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
			
			IF (@m_FieldType = 'Float') set @select = @tbl + '.FloatValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Float') set @join = 'Left Outer Join v_MPP_PositionTypeValue ' + @tbl + ' On ' + @tbl + '.PositionTypeRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
		
			IF (@m_FieldType in ('FlatStructure','TreeStructure')) set @select = @tbl + '.Title ' + @col + '_Title, ' + @tbl + '.Description ' + @col + '_Description, ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType in ('FlatStructure','TreeStructure')) set @join = 'Left Outer Join v_MPP_PositionTypeValue ppv' + @tbl + ' On ppv' + @tbl + '.PositionTypeRef = ' + @alias + '.' + @RefColumn + ' And ppv' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + ''' Left Outer Join v_Project_Node ' + @tbl + ' On ppv' + @tbl + '.NodeRef = ' + @tbl + '.PrimKey ' + CHAR(13)+CHAR(10)
		
			IF (@m_FieldType = 'OrganizationNode') set @select = @tbl + '.Title ' + @col + '_Title, ' + @tbl + '.Description ' + @col + '_Description, ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'OrganizationNode') set @join = 'Left Outer Join v_MPP_PositionTypeValue ppv' + @tbl + ' On ppv' + @tbl + '.PositionTypeRef = ' + @alias + '.' + @RefColumn + ' And ppv' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + ''' Left Outer Join v_Project_OrganizationUnit ' + @tbl + ' On ppv' + @tbl + '.OrganizationUnitValueRef = ' + @tbl + '.PrimKey ' + CHAR(13)+CHAR(10)
		
			IF (@m_FieldType = 'Person') set @select = @tbl + '.PersonID ' + @col + '_PersonID, ' + @tbl + '.LastName ' + @col + 'LastName, ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Person') set @join = 'Left Outer Join v_MPP_PositionTypeValue ppv' + @tbl + ' On ppv' + @tbl + '.PositionTypeRef = ' + @alias + '.' + @RefColumn + ' And ppv' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + ''' Left Outer Join v_Project_Person ' + @tbl + ' On ppv' + @tbl + '.PersonValueRef = ' + @tbl + '.PrimKey ' + CHAR(13)+CHAR(10)
		
			Set @allSelects = @allSelects + @select
			Set @allJoins = @allJoins + @join
		
			insert into @outputTable( FieldList, LeftJoin ) values ( @select, @join )		
			Fetch Next From my_cursor Into @m_FieldTitle, @m_FieldType, @m_FieldRef
		End
		Close my_cursor
		Deallocate my_cursor		

	END
	
	IF (@TableName = 't_MPP_ProjectPosition')
	BEGIN
		Declare my_cursor Cursor For
		--- Collect custom fields used for project positions
		select f.title, f.[type], ppf.FieldRef
		from t_MPP_ProjectPositionField ppf
		--inner join t_Project_ProjectRevision pr on ppf.ProjectRevisionRef = pr.PrimKey
		inner join t_Project_Revision r on ppf.RevisionRef = r.PrimKey
		left outer join t_Project_Field f on ppf.FieldRef = f.PrimKey
		where r.CurrentRevision = 1
		group by f.title, f.[type], ppf.FieldRef
		order by f.title, f.[type], ppf.FieldRef
	
		set @col_prefix = 'CF_' + @alias + '_'
		set @tbl_prefix = 't_'
		
		set @cf_count = 200
	
		Open my_cursor
		Fetch Next From my_cursor Into @m_FieldTitle, @m_FieldType, @m_FieldRef
		While @@FETCH_STATUS = 0
		Begin
			set @cf_count = @cf_count + 1
			While PatIndex(@fieldtitlepattern, @m_FieldTitle) > 0 Set @m_FieldTitle = Stuff(@m_FieldTitle, PatIndex(@fieldtitlepattern, @m_FieldTitle), 1, '')
	
			set @tbl = @tbl_prefix +  convert(varchar,@cf_count)
			set @col = @col_prefix + @m_FieldTitle + '_' + convert(varchar,@cf_count)

			IF (@m_FieldType = 'Date') set @select = @tbl + '.DateTimeValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Date') set @join = 'Left Outer Join v_MPP_ProjectPositionValue ' + @tbl + ' On ' + @tbl + '.ProjectPositionRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
	
			IF (@m_FieldType = 'Int') set @select = @tbl + '.IntValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Int') set @join = 'Left Outer Join v_MPP_ProjectPositionValue ' + @tbl + ' On ' + @tbl + '.ProjectPositionRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
	
			IF (@m_FieldType = 'Text') set @select = @tbl + '.TextValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Text') set @join = 'Left Outer Join v_MPP_ProjectPositionValue ' + @tbl + ' On ' + @tbl + '.ProjectPositionRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
			
			IF (@m_FieldType = 'Float') set @select = @tbl + '.FloatValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Float') set @join = 'Left Outer Join v_MPP_ProjectPositionValue ' + @tbl + ' On ' + @tbl + '.ProjectPositionRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
		
			IF (@m_FieldType in ('FlatStructure','TreeStructure')) set @select = @tbl + '.Title ' + @col + '_Title, ' + @tbl + '.Description ' + @col + '_Description, ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType in ('FlatStructure','TreeStructure')) set @join = 'Left Outer Join v_MPP_ProjectPositionValue ppv' + @tbl + ' On ppv' + @tbl + '.ProjectPositionRef = ' + @alias + '.' + @RefColumn + ' And ppv' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + ''' Left Outer Join v_Project_Node ' + @tbl + ' On ppv' + @tbl + '.NodeRef = ' + @tbl + '.PrimKey ' + CHAR(13)+CHAR(10)
		
			IF (@m_FieldType = 'OrganizationNode') set @select = @tbl + '.Title ' + @col + '_Title, ' + @tbl + '.Description ' + @col + '_Description, ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'OrganizationNode') set @join = 'Left Outer Join v_MPP_ProjectPositionValue ppv' + @tbl + ' On ppv' + @tbl + '.ProjectPositionRef = ' + @alias + '.' + @RefColumn + ' And ppv' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + ''' Left Outer Join v_Project_OrganizationUnit ' + @tbl + ' On ppv' + @tbl + '.OrganizationUnitValueRef = ' + @tbl + '.PrimKey ' + CHAR(13)+CHAR(10)
		
			IF (@m_FieldType = 'Person') set @select = @tbl + '.PersonID ' + @col + '_PersonID, ' + @tbl + '.LastName ' + @col + 'LastName, ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Person') set @join = 'Left Outer Join v_MPP_ProjectPositionValue ppv' + @tbl + ' On ppv' + @tbl + '.ProjectPositionRef = ' + @alias + '.' + @RefColumn + ' And ppv' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + ''' Left Outer Join v_Project_Person ' + @tbl + ' On ppv' + @tbl + '.PersonValueRef = ' + @tbl + '.PrimKey ' + CHAR(13)+CHAR(10)
		
			Set @allSelects = @allSelects + @select
			Set @allJoins = @allJoins + @join
		
			insert into @outputTable( FieldList, LeftJoin ) values ( @select, @join )		
			Fetch Next From my_cursor Into @m_FieldTitle, @m_FieldType, @m_FieldRef
		End
		Close my_cursor
		Deallocate my_cursor		
	END

	IF (@TableName = 't_Project_ExpMhrs')
	BEGIN
		Declare my_cursor Cursor For
		--- Collect custom fields used for expended man-hours
		select f.title, f.[type], emf.FieldRef
		from t_Project_ExpMhrsField emf
		inner join t_Project_Revision r on emf.RevisionRef = r.PrimKey
		left outer join t_Project_Field f on emf.FieldRef = f.PrimKey
		where r.CurrentRevision = 1
		group by f.title, f.[type], emf.FieldRef
		order by f.title, f.[type], emf.FieldRef
	
		set @col_prefix = 'CF_' + @alias + '_'
		set @tbl_prefix = 't_'
		
		set @cf_count = 300
	
		Open my_cursor
		Fetch Next From my_cursor Into @m_FieldTitle, @m_FieldType, @m_FieldRef
		While @@FETCH_STATUS = 0
		Begin
			set @cf_count = @cf_count + 1
			While PatIndex(@fieldtitlepattern, @m_FieldTitle) > 0 Set @m_FieldTitle = Stuff(@m_FieldTitle, PatIndex(@fieldtitlepattern, @m_FieldTitle), 1, '')
	
			set @tbl = @tbl_prefix +  convert(varchar,@cf_count)
			set @col = @col_prefix + @m_FieldTitle + '_' + convert(varchar,@cf_count)

			IF (@m_FieldType = 'Date') set @select = @tbl + '.DateTimeValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Date') set @join = 'Left Outer Join v_Project_ExpMhrsValue ' + @tbl + ' On ' + @tbl + '.ExpMhrsRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
	
			IF (@m_FieldType = 'Int') set @select = @tbl + '.IntValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Int') set @join = 'Left Outer Join v_Project_ExpMhrsValue ' + @tbl + ' On ' + @tbl + '.ExpMhrsRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
	
			IF (@m_FieldType = 'Text') set @select = @tbl + '.TextValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Text') set @join = 'Left Outer Join v_Project_ExpMhrsValue ' + @tbl + ' On ' + @tbl + '.ExpMhrsRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
			
			IF (@m_FieldType = 'Float') set @select = @tbl + '.FloatValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Float') set @join = 'Left Outer Join v_Project_ExpMhrsValue ' + @tbl + ' On ' + @tbl + '.ExpMhrsRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
		
			IF (@m_FieldType in ('FlatStructure','TreeStructure')) set @select = @tbl + '.Title ' + @col + '_Title, ' + @tbl + '.Description ' + @col + '_Description, ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType in ('FlatStructure','TreeStructure')) set @join = 'Left Outer Join v_Project_ExpMhrsValue ppv' + @tbl + ' On ppv' + @tbl + '.ExpMhrsRef = ' + @alias + '.' + @RefColumn + ' And ppv' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + ''' Left Outer Join v_Project_Node ' + @tbl + ' On ppv' + @tbl + '.NodeRef = ' + @tbl + '.PrimKey ' + CHAR(13)+CHAR(10)
		
			IF (@m_FieldType = 'OrganizationNode') set @select = @tbl + '.Title ' + @col + '_Title, ' + @tbl + '.Description ' + @col + '_Description, ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'OrganizationNode') set @join = 'Left Outer Join v_Project_ExpMhrsValue ppv' + @tbl + ' On ppv' + @tbl + '.ExpMhrsRef = ' + @alias + '.' + @RefColumn + ' And ppv' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + ''' Left Outer Join v_Project_OrganizationUnit ' + @tbl + ' On ppv' + @tbl + '.OrganizationUnitValueRef = ' + @tbl + '.PrimKey ' + CHAR(13)+CHAR(10)
		
			IF (@m_FieldType = 'Person') set @select = @tbl + '.PersonID ' + @col + '_PersonID, ' + @tbl + '.LastName ' + @col + 'LastName, ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Person') set @join = 'Left Outer Join v_Project_ExpMhrsValue ppv' + @tbl + ' On ppv' + @tbl + '.ExpMhrsRef = ' + @alias + '.' + @RefColumn + ' And ppv' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + ''' Left Outer Join v_Project_Person ' + @tbl + ' On ppv' + @tbl + '.PersonValueRef = ' + @tbl + '.PrimKey ' + CHAR(13)+CHAR(10)
		
			Set @allSelects = @allSelects + @select
			Set @allJoins = @allJoins + @join
		
			insert into @outputTable( FieldList, LeftJoin ) values ( @select, @join )		
			Fetch Next From my_cursor Into @m_FieldTitle, @m_FieldType, @m_FieldRef
		End
		Close my_cursor
		Deallocate my_cursor		
	END
	
	IF (@TableName = 't_Project_OrganizationUnit')
	BEGIN
		Declare my_cursor Cursor For
		--- Collect custom fields used for organization units
		select f.title, f.[type], ouf.FieldRef
		from t_Project_OrganizationUnitField ouf
		inner join t_Project_Revision r on ouf.RevisionRef = r.PrimKey
		left outer join t_Project_Field f on ouf.FieldRef = f.PrimKey
		where r.CurrentRevision = 1
		group by f.title, f.[type], ouf.FieldRef
		order by f.title, f.[type], ouf.FieldRef
	
		set @col_prefix = 'CF_' + @alias + '_'
		set @tbl_prefix = 't'
		
		set @cf_count = 400
	
		Open my_cursor
		Fetch Next From my_cursor Into @m_FieldTitle, @m_FieldType, @m_FieldRef
		While @@FETCH_STATUS = 0
		Begin
			set @cf_count = @cf_count + 1
			While PatIndex(@fieldtitlepattern, @m_FieldTitle) > 0 Set @m_FieldTitle = Stuff(@m_FieldTitle, PatIndex(@fieldtitlepattern, @m_FieldTitle), 1, '')
	
			set @tbl = @tbl_prefix + convert(varchar,@cf_count)
			set @col = @col_prefix + @m_FieldTitle --+ '_' + convert(varchar,@cf_count)

			IF (@m_FieldType = 'Date') set @select = @tbl + '.DateTimeValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Date') set @join = 'Left Outer Join v_Project_OrganizationUnitValue ' + @tbl + ' On ' + @tbl + '.OrganizationUnitRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
	
			IF (@m_FieldType = 'Int') set @select = @tbl + '.IntValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Int') set @join = 'Left Outer Join v_Project_OrganizationUnitValue ' + @tbl + ' On ' + @tbl + '.OrganizationUnitRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
	
			IF (@m_FieldType = 'Text') set @select = @tbl + '.TextValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Text') set @join = 'Left Outer Join v_Project_OrganizationUnitValue ' + @tbl + ' On ' + @tbl + '.OrganizationUnitRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
			
			IF (@m_FieldType = 'Float') set @select = @tbl + '.FloatValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Float') set @join = 'Left Outer Join v_Project_OrganizationUnitValue ' + @tbl + ' On ' + @tbl + '.OrganizationUnitRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
		
			IF (@m_FieldType in ('FlatStructure','TreeStructure')) set @select = @tbl + '.Title ' + @col + '_Title, ' + @tbl + '.Description ' + @col + '_Description, ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType in ('FlatStructure','TreeStructure')) set @join = 'Left Outer Join v_Project_OrganizationUnitValue ppv' + @tbl + ' On ppv' + @tbl + '.OrganizationUnitRef = ' + @alias + '.' + @RefColumn + ' And ppv' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + ''' Left Outer Join v_Project_Node ' + @tbl + ' On ppv' + @tbl + '.NodeRef = ' + @tbl + '.PrimKey ' + CHAR(13)+CHAR(10)
		
			IF (@m_FieldType = 'OrganizationNode') set @select = @tbl + '.Title ' + @col + '_Title, ' + @tbl + '.Description ' + @col + '_Description, ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'OrganizationNode') set @join = 'Left Outer Join v_Project_OrganizationUnitValue ppv' + @tbl + ' On ppv' + @tbl + '.OrganizationUnitRef = ' + @alias + '.' + @RefColumn + ' And ppv' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + ''' Left Outer Join v_Project_OrganizationUnit ' + @tbl + ' On ppv' + @tbl + '.OrganizationUnitValueRef = ' + @tbl + '.PrimKey ' + CHAR(13)+CHAR(10)
		
			IF (@m_FieldType = 'Person') set @select = @tbl + '.PersonID ' + @col + '_PersonID, ' + @tbl + '.LastName ' + @col + 'LastName, ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Person') set @join = 'Left Outer Join v_Project_OrganizationUnitValue ppv' + @tbl + ' On ppv' + @tbl + '.OrganizationUnitRef = ' + @alias + '.' + @RefColumn + ' And ppv' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + ''' Left Outer Join v_Project_Person ' + @tbl + ' On ppv' + @tbl + '.PersonValueRef = ' + @tbl + '.PrimKey ' + CHAR(13)+CHAR(10)
		
			Set @allSelects = @allSelects + @select
			Set @allJoins = @allJoins + @join
		
			insert into @outputTable( FieldList, LeftJoin ) values ( @select, @join )		
			Fetch Next From my_cursor Into @m_FieldTitle, @m_FieldType, @m_FieldRef
		End
		Close my_cursor
		Deallocate my_cursor

	END
	
	IF (@TableName = 't_Project_Person')
	BEGIN
		Declare my_cursor Cursor For
		--- Collect custom fields used for persons
		select f.title, f.[type], pf.FieldRef
		from t_Project_PersonField pf
		inner join t_Project_Revision r on pf.RevisionRef = r.PrimKey
		left outer join t_Project_Field f on pf.FieldRef = f.PrimKey
		where r.CurrentRevision = 1
		group by f.title, f.[type], pf.FieldRef
		order by f.title, f.[type], pf.FieldRef
	
		set @col_prefix = 'CF_' + @alias + '_'
		set @tbl_prefix = 't'
		
		set @cf_count = 500
	
		Open my_cursor
		Fetch Next From my_cursor Into @m_FieldTitle, @m_FieldType, @m_FieldRef
		While @@FETCH_STATUS = 0
		Begin
			set @cf_count = @cf_count + 1
			While PatIndex(@fieldtitlepattern, @m_FieldTitle) > 0 Set @m_FieldTitle = Stuff(@m_FieldTitle, PatIndex(@fieldtitlepattern, @m_FieldTitle), 1, '')
	
			set @tbl = @tbl_prefix + convert(varchar,@cf_count)
			set @col = @col_prefix + @m_FieldTitle --+ '_' + convert(varchar,@cf_count)

			IF (@m_FieldType = 'Date') set @select = @tbl + '.DateTimeValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Date') set @join = 'Left Outer Join v_Project_PersonValue ' + @tbl + ' On ' + @tbl + '.PersonRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
	
			IF (@m_FieldType = 'Int') set @select = @tbl + '.IntValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Int') set @join = 'Left Outer Join v_Project_PersonValue ' + @tbl + ' On ' + @tbl + '.PersonRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
	
			IF (@m_FieldType = 'Text') set @select = @tbl + '.TextValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Text') set @join = 'Left Outer Join v_Project_PersonValue ' + @tbl + ' On ' + @tbl + '.PersonRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
			
			IF (@m_FieldType = 'Float') set @select = @tbl + '.FloatValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Float') set @join = 'Left Outer Join v_Project_PersonValue ' + @tbl + ' On ' + @tbl + '.PersonRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
		
			IF (@m_FieldType in ('FlatStructure','TreeStructure')) set @select = @tbl + '.Title ' + @col + '_Title, ' + @tbl + '.Description ' + @col + '_Description, ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType in ('FlatStructure','TreeStructure')) set @join = 'Left Outer Join v_Project_PersonValue ppv' + @tbl + ' On ppv' + @tbl + '.PersonRef = ' + @alias + '.' + @RefColumn + ' And ppv' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + ''' Left Outer Join v_Project_Node ' + @tbl + ' On ppv' + @tbl + '.NodeRef = ' + @tbl + '.PrimKey ' + CHAR(13)+CHAR(10)
		
			IF (@m_FieldType = 'OrganizationNode') set @select = @tbl + '.Title ' + @col + '_Title, ' + @tbl + '.Description ' + @col + '_Description, ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'OrganizationNode') set @join = 'Left Outer Join v_Project_PersonValue ppv' + @tbl + ' On ppv' + @tbl + '.PersonRef = ' + @alias + '.' + @RefColumn + ' And ppv' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + ''' Left Outer Join v_Project_OrganizationUnit ' + @tbl + ' On ppv' + @tbl + '.OrganizationUnitValueRef = ' + @tbl + '.PrimKey ' + CHAR(13)+CHAR(10)
		
			IF (@m_FieldType = 'Person') set @select = @tbl + '.PersonID ' + @col + '_PersonID, ' + @tbl + '.LastName ' + @col + 'LastName, ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Person') set @join = 'Left Outer Join v_Project_PersonValue ppv' + @tbl + ' On ppv' + @tbl + '.PersonRef = ' + @alias + '.' + @RefColumn + ' And ppv' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + ''' Left Outer Join v_Project_Person ' + @tbl + ' On ppv' + @tbl + '.PersonValueRef = ' + @tbl + '.PrimKey ' + CHAR(13)+CHAR(10)
		
			Set @allSelects = @allSelects + @select
			Set @allJoins = @allJoins + @join
		
			insert into @outputTable( FieldList, LeftJoin ) values ( @select, @join )		
			Fetch Next From my_cursor Into @m_FieldTitle, @m_FieldType, @m_FieldRef
		End
		Close my_cursor
		Deallocate my_cursor

	END
	
	IF (@TableName = 't_Project_ProjectRevision')
	BEGIN

		Declare my_cursor Cursor For
		--- Collect custom fields used for project positions
		select f.title, f.[type], prf.FieldRef
		from t_Project_ProjectRevisionField prf
		inner join t_Project_Revision r on prf.RevisionRef = r.PrimKey
		left outer join t_Project_Field f on prf.FieldRef = f.PrimKey
		where r.CurrentRevision = 1
		group by f.title, f.[type], prf.FieldRef
		order by f.title, f.[type], prf.FieldRef
	
		set @col_prefix = 'CF_' + @alias + '_'
		set @tbl_prefix = 't_'
		
		set @cf_count = 600
	
		Open my_cursor
		Fetch Next From my_cursor Into @m_FieldTitle, @m_FieldType, @m_FieldRef
		While @@FETCH_STATUS = 0
		Begin
			set @cf_count = @cf_count + 1
			While PatIndex(@fieldtitlepattern, @m_FieldTitle) > 0 Set @m_FieldTitle = Stuff(@m_FieldTitle, PatIndex(@fieldtitlepattern, @m_FieldTitle), 1, '')
	
			set @tbl = @tbl_prefix +  convert(varchar,@cf_count)
			set @col = @col_prefix + @m_FieldTitle + '_' + convert(varchar,@cf_count)

			IF (@m_FieldType = 'Date') set @select = @tbl + '.DateTimeValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Date') set @join = 'Left Outer Join v_Project_ProjectRevisionValue ' + @tbl + ' On ' + @tbl + '.ProjectRevisionRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
	
			IF (@m_FieldType = 'Int') set @select = @tbl + '.IntValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Int') set @join = 'Left Outer Join v_Project_ProjectRevisionValue ' + @tbl + ' On ' + @tbl + '.ProjectRevisionRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
	
			IF (@m_FieldType = 'Text') set @select = @tbl + '.TextValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Text') set @join = 'Left Outer Join v_Project_ProjectRevisionValue ' + @tbl + ' On ' + @tbl + '.ProjectRevisionRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
			
			IF (@m_FieldType = 'Float') set @select = @tbl + '.FloatValue ' + @col + ', ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Float') set @join = 'Left Outer Join v_Project_ProjectRevisionValue ' + @tbl + ' On ' + @tbl + '.ProjectRevisionRef = ' + @alias + '.' + @RefColumn + ' And ' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + '''' + CHAR(13)+CHAR(10)
		
			IF (@m_FieldType in ('FlatStructure','TreeStructure')) set @select = @tbl + '.Title ' + @col + '_Title, ' + @tbl + '.Description ' + @col + '_Description, ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType in ('FlatStructure','TreeStructure')) set @join = 'Left Outer Join v_Project_ProjectRevisionValue ppv' + @tbl + ' On ppv' + @tbl + '.ProjectRevisionRef = ' + @alias + '.' + @RefColumn + ' And ppv' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + ''' Left Outer Join v_Project_Node ' + @tbl + ' On ppv' + @tbl + '.NodeRef = ' + @tbl + '.PrimKey ' + CHAR(13)+CHAR(10)
		
			IF (@m_FieldType = 'OrganizationNode') set @select = @tbl + '.Title ' + @col + '_Title, ' + @tbl + '.Description ' + @col + '_Description, ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'OrganizationNode') set @join = 'Left Outer Join v_Project_ProjectRevisionValue ppv' + @tbl + ' On ppv' + @tbl + '.ProjectRevisionRef = ' + @alias + '.' + @RefColumn + ' And ppv' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + ''' Left Outer Join v_Project_OrganizationUnit ' + @tbl + ' On ppv' + @tbl + '.OrganizationUnitValueRef = ' + @tbl + '.PrimKey ' + CHAR(13)+CHAR(10)
		
			IF (@m_FieldType = 'Person') set @select = @tbl + '.PersonID ' + @col + '_PersonID, ' + @tbl + '.LastName ' + @col + 'LastName, ' + CHAR(13)+CHAR(10)
			IF (@m_FieldType = 'Person') set @join = 'Left Outer Join v_Project_ProjectRevisionValue ppv' + @tbl + ' On ppv' + @tbl + '.ProjectRevisionRef = ' + @alias + '.' + @RefColumn + ' And ppv' + @tbl + '.FieldRef = ''' + convert(varchar(36),@m_FieldRef) + ''' Left Outer Join v_Project_Person ' + @tbl + ' On ppv' + @tbl + '.PersonValueRef = ' + @tbl + '.PrimKey ' + CHAR(13)+CHAR(10)
		
			Set @allSelects = @allSelects + @select
			Set @allJoins = @allJoins + @join
		
			insert into @outputTable( FieldList, LeftJoin ) values ( @select, @join )		
			Fetch Next From my_cursor Into @m_FieldTitle, @m_FieldType, @m_FieldRef
		End
		Close my_cursor
		Deallocate my_cursor		

	END
	
	Print @errmsg
	
	select * from @outputTable
END
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Altering [dbo].[t_Reporting_Dashboard]'
GO
ALTER TABLE [dbo].[t_Reporting_Dashboard] ADD
[ThumbnailImage] [varbinary] (max) NULL
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[v_Reporting_Dashboard]'
GO
EXEC sp_refreshview N'[dbo].[v_Reporting_Dashboard]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating [dbo].[t_Reporting_DashboardTag]'
GO
CREATE TABLE [dbo].[t_Reporting_DashboardTag]
(
[DashboardRef] [uniqueidentifier] NOT NULL,
[Tag] [nvarchar] (50) COLLATE Latin1_General_CI_AS NOT NULL,
[PrimKey] [uniqueidentifier] NOT NULL CONSTRAINT [DF_t_Reporting_DashboardTag_PrimKey] DEFAULT (newsequentialid()),
[TimeStamp] [timestamp] NOT NULL
)
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating primary key [PK_t_Reporting_DashboardTag] on [dbo].[t_Reporting_DashboardTag]'
GO
ALTER TABLE [dbo].[t_Reporting_DashboardTag] ADD CONSTRAINT [PK_t_Reporting_DashboardTag] PRIMARY KEY CLUSTERED  ([PrimKey])
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating [dbo].[v_Reporting_DashboardTag]'
GO

CREATE VIEW [dbo].[v_Reporting_DashboardTag]
AS
SELECT * FROM [t_Reporting_DashboardTag] with (nolock)
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating trigger [dbo].[tri_Reporting_DashboardTag] on [dbo].[t_Reporting_DashboardTag]'
GO

CREATE TRIGGER [dbo].[tri_Reporting_DashboardTag] 
ON [dbo].[t_Reporting_DashboardTag]
AFTER INSERT
AS

-- Prevent client from seing number of records affected
SET NOCOUNT ON

/* <PermissionCheck> */
    
/* </PermissionCheck> */

/* <CustomSection> */

/* </CustomSection> */

    GOTO TriggerContinue

TriggerError:
    -- Log error
    DECLARE @UserName AS NVARCHAR(128)
    SET @UserName = suser_sname()

    DECLARE @ErrorMessage AS NVARCHAR(512)
    SET @ErrorMessage = N'No permissions to INSERT data [] for user [' + @UserName + ']'

    raiserror (@ErrorMessage, 14, 23)
    GOTO TriggerRollback

TriggerRollback:
    ROLLBACK TRANSACTION
    GOTO TriggerEnd

TriggerContinue:

    IF NOT OBJECT_ID('v_Security_DataLog') IS NULL
    BEGIN
        INSERT INTO v_Security_DataLog([Table], [Reference], [User], [Operation], [Time])
        SELECT 't_Reporting_DashboardTag', PrimKey, suser_sname(), 'I', getdate()
        FROM inserted
    END


TriggerEnd:

    IF NOT @ErrorMessage IS NULL
    BEGIN
        IF NOT OBJECT_ID('v_Security_ErrorLog') IS NULL
        BEGIN
            INSERT INTO v_Security_ErrorLog(Message, Object, Operation, Login)
            SELECT @ErrorMessage, 't_Reporting_DashboardTag', 'INSERT', @UserName
        END
    END

   
    RETURN
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating trigger [dbo].[tru_Reporting_DashboardTag] on [dbo].[t_Reporting_DashboardTag]'
GO

CREATE TRIGGER [dbo].[tru_Reporting_DashboardTag] 
ON [dbo].[t_Reporting_DashboardTag]
AFTER UPDATE
AS

-- Prevent client from seing number of records affected
SET NOCOUNT ON

/* <PermissionCheck> */
    
/* </PermissionCheck> */

/* <CustomSection> */

/* </CustomSection> */

    GOTO TriggerContinue

TriggerError:
    -- Log error
    DECLARE @UserName AS NVARCHAR(128)
    SET @UserName = suser_sname()

    DECLARE @ErrorMessage AS NVARCHAR(512)
    SET @ErrorMessage = N'No permissions to UPDATE data [] for user [' + @UserName + ']'

    raiserror (@ErrorMessage, 14, 23)
    GOTO TriggerRollback

TriggerRollback:
    ROLLBACK TRANSACTION
    GOTO TriggerEnd

TriggerContinue:

    IF NOT OBJECT_ID('v_Security_DataLog') IS NULL
    BEGIN
        INSERT INTO v_Security_DataLog([Table], [Reference], [User], [Operation], [Time])
        SELECT 't_Reporting_DashboardTag', PrimKey, suser_sname(), 'U', getdate()
        FROM inserted
    END


TriggerEnd:

    IF NOT @ErrorMessage IS NULL
    BEGIN
        IF NOT OBJECT_ID('v_Security_ErrorLog') IS NULL
        BEGIN
            INSERT INTO v_Security_ErrorLog(Message, Object, Operation, Login)
            SELECT @ErrorMessage, 't_Reporting_DashboardTag', 'UPDATE', @UserName
        END
    END

   
    RETURN
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating trigger [dbo].[trd_Reporting_DashboardTag] on [dbo].[t_Reporting_DashboardTag]'
GO

CREATE TRIGGER [dbo].[trd_Reporting_DashboardTag] 
ON [dbo].[t_Reporting_DashboardTag]
AFTER DELETE
AS

-- Prevent client from seing number of records affected
SET NOCOUNT ON

/* <PermissionCheck> */
    
/* </PermissionCheck> */

/* <CustomSection> */

/* </CustomSection> */

    GOTO TriggerContinue

TriggerError:
    -- Log error
    DECLARE @UserName AS NVARCHAR(128)
    SET @UserName = suser_sname()

    DECLARE @ErrorMessage AS NVARCHAR(512)
    SET @ErrorMessage = N'No permissions to DELETE data [] for user [' + @UserName + ']'

    raiserror (@ErrorMessage, 14, 23)
    GOTO TriggerRollback

TriggerRollback:
    ROLLBACK TRANSACTION
    GOTO TriggerEnd

TriggerContinue:

    IF NOT OBJECT_ID('v_Security_DataLog') IS NULL
    BEGIN
        INSERT INTO v_Security_DataLog([Table], [Reference], [User], [Operation], [Time])
        SELECT 't_Reporting_DashboardTag', PrimKey, suser_sname(), 'D', getdate()
        FROM deleted
    END


TriggerEnd:

    IF NOT @ErrorMessage IS NULL
    BEGIN
        IF NOT OBJECT_ID('v_Security_ErrorLog') IS NULL
        BEGIN
            INSERT INTO v_Security_ErrorLog(Message, Object, Operation, Login)
            SELECT @ErrorMessage, 't_Reporting_DashboardTag', 'DELETE', @UserName
        END
    END

   
    RETURN
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Altering [dbo].[vc_Project_MyPerson]'
GO

ALTER VIEW [dbo].[vc_Project_MyPerson]
AS
SELECT 
OrganizationUnitRef,
RevisionRef,
Login,
PersonID,
Firstname,
Lastname,
CalendarRef,
HiredFrom,
HiredTo,
UserRef,
ImportID,
Email,
PrimKey,
TimeStamp 
FROM v_Project_Person
WHERE [LOGIN] = SUSER_SNAME() AND RevisionRef = (SELECT TOP 1 PrimKey FROM v_Project_Revision WHERE CurrentRevision = 1)
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[v_Project_ExpMhrs]'
GO
EXEC sp_refreshview N'[dbo].[v_Project_ExpMhrs]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[vc_Reporting_DataSourceUsage]'
GO
EXEC sp_refreshview N'[dbo].[vc_Reporting_DataSourceUsage]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[v_Project_ExpMhrsValue]'
GO
EXEC sp_refreshview N'[dbo].[v_Project_ExpMhrsValue]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[v_Project_Node]'
GO
EXEC sp_refreshview N'[dbo].[v_Project_Node]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[vc_DataSource_PersonWithAvailability]'
GO
EXEC sp_refreshview N'[dbo].[vc_DataSource_PersonWithAvailability]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[v_Project_OrganizationUnitValue]'
GO
EXEC sp_refreshview N'[dbo].[v_Project_OrganizationUnitValue]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[vc_Project_OrganizationUnitLookup]'
GO
EXEC sp_refreshview N'[dbo].[vc_Project_OrganizationUnitLookup]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[v_MPP_ProjectPositionLoad]'
GO
EXEC sp_refreshview N'[dbo].[v_MPP_ProjectPositionLoad]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Altering [dbo].[vc_DataSource_ProjectPositionWithLoad]'
GO

ALTER VIEW [dbo].[vc_DataSource_ProjectPositionWithLoad]
AS

select 
	--- Standard columns (Position)
	pos.PositionID, pos.Title, pos.[Description], 
	pos.ProjectRevisionRef, pos.OrganizationUnitPositionTypeRef, pos.PersonRef, pos.CalendarRef, pos.PrimKey, pos.[TimeStamp],

	--- Related columns (Project)
	proj.ProjectCode, proj.Title ProjectTitle,  proj.[Description] ProjectDescription,
	proj.ProjectTypeRef, proj.ProjectStatusRef, 
	
	--- Related columns (Project revision)
	projrev.Title As ProjectRevisionTitle, projrev.[Description] As ProjectRevisionDescription, 
	projrev.ProjectRef, 

	--- Related columns (Calendar)
	cal.Title CalendarTitle, cal.[Description] CalendarDescription,

	--- Related column (Position type)
	pt.Title PosTypeTitle, pt.[Description] PosTypeDescription,

	--- Related colum (Organisation unit for POSITION)
	oupt.OrganizationUnitRef, oupt.PositionTypeRef,
	ou.OrganizationUnitCode, ou.Title OrganizationTitle, ou.[Description] OrganizationDescription,

	--- Related column (Person)
	pers.PersonID, pers.Firstname, pers.Lastname,
	isnull(pers.Firstname,'') + ' ' + isnull(pers.Lastname,'') PersonName,
	isnull(pers.PersonID,'') + ' - ' + isnull(pers.Firstname,'') + ' ' + isnull(pers.Lastname,'') PersonIDAndName,
	
	--- Related colum (Organisation unit for PERSON)
	pers.OrganizationUnitRef PersonOrganizationUnitRef,
	ox.OrganizationUnitCode PersonOrganizationUnitCode, ox.Title PersonOrganizationTitle, ox.[Description] PersonOrganizationDescription,

	--- Related column (Project type)
	projtype.Title ProjectTypeTitle, projtype.[Description] ProjectTypeDescription,

	--- Related column (Project status)
	projstat.Title ProjectStatusTitle, projstat.[Description] ProjectStatusDescription,

	pload.[From], pload.[To], pload.[Load]

from 
	dbo.v_Project_Revision rev with(nolock)
	inner join dbo.v_Project_ProjectRevision projrev with(nolock) on rev.PrimKey = projrev.RevisionRef
	inner join dbo.v_Project_Project proj with(nolock) on projrev.ProjectRef = proj.PrimKey and projrev.PrimKey = proj.LiveProjectRevisionRef
	inner join dbo.v_MPP_ProjectPosition pos with(nolock) on proj.LiveProjectRevisionRef = pos.ProjectRevisionRef
	inner join dbo.v_MPP_ProjectPositionLoad pload with(nolock) on pload.ProjectPositionRef = pos.PrimKey
	left outer join dbo.v_Project_Person pers with(nolock) on pers.RevisionRef = rev.PrimKey and pers.PrimKey = pos.PersonRef
	left outer join dbo.v_Project_Calendar cal with(nolock) on cal.RevisionRef = rev.PrimKey and cal.PrimKey = pos.CalendarRef
	left outer join dbo.v_MPP_OrganizationUnitPositionType oupt with(nolock) on pos.OrganizationUnitPositionTypeRef = oupt.PrimKey
	left outer join dbo.v_MPP_PositionType pt with(nolock) on oupt.PositionTypeRef = pt.PrimKey 
	left outer join dbo.v_Project_OrganizationUnit ou with(nolock) on ou.PrimKey = oupt.OrganizationUnitRef
	left outer join dbo.v_Project_OrganizationUnit ox with(nolock) on ox.PrimKey = pers.OrganizationUnitRef
	left outer join dbo.v_Project_ProjectType projtype with(nolock) on projtype.PrimKey = proj.ProjectTypeRef
	left outer join dbo.v_Project_ProjectStatus projstat with(nolock) on projstat.PrimKey = proj.ProjectStatusRef

Where 
	rev.CurrentRevision = 1





GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating [dbo].[t_Reporting_UserDashboard]'
GO
CREATE TABLE [dbo].[t_Reporting_UserDashboard]
(
[DashboardRef] [uniqueidentifier] NOT NULL,
[UserRef] [uniqueidentifier] NOT NULL,
[PrimKey] [uniqueidentifier] NOT NULL CONSTRAINT [DF_t_Reporting_UserDashboard_PrimKey] DEFAULT (newsequentialid()),
[TimeStamp] [timestamp] NOT NULL
)
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating primary key [PK_t_Reporting_UserDashboard] on [dbo].[t_Reporting_UserDashboard]'
GO
ALTER TABLE [dbo].[t_Reporting_UserDashboard] ADD CONSTRAINT [PK_t_Reporting_UserDashboard] PRIMARY KEY CLUSTERED  ([PrimKey])
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating [dbo].[v_Reporting_UserDashboard]'
GO

CREATE VIEW [dbo].[v_Reporting_UserDashboard]
AS
SELECT * FROM [t_Reporting_UserDashboard] with (nolock)
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating trigger [dbo].[tri_Reporting_UserDashboard] on [dbo].[t_Reporting_UserDashboard]'
GO

CREATE TRIGGER [dbo].[tri_Reporting_UserDashboard] 
ON [dbo].[t_Reporting_UserDashboard]
AFTER INSERT
AS

-- Prevent client from seing number of records affected
SET NOCOUNT ON

/* <PermissionCheck> */
    
/* </PermissionCheck> */

/* <CustomSection> */

/* </CustomSection> */

    GOTO TriggerContinue

TriggerError:
    -- Log error
    DECLARE @UserName AS NVARCHAR(128)
    SET @UserName = suser_sname()

    DECLARE @ErrorMessage AS NVARCHAR(512)
    SET @ErrorMessage = N'No permissions to INSERT data [] for user [' + @UserName + ']'

    raiserror (@ErrorMessage, 14, 23)
    GOTO TriggerRollback

TriggerRollback:
    ROLLBACK TRANSACTION
    GOTO TriggerEnd

TriggerContinue:

    IF NOT OBJECT_ID('v_Security_DataLog') IS NULL
    BEGIN
        INSERT INTO v_Security_DataLog([Table], [Reference], [User], [Operation], [Time])
        SELECT 't_Reporting_UserDashboard', PrimKey, suser_sname(), 'I', getdate()
        FROM inserted
    END


TriggerEnd:

    IF NOT @ErrorMessage IS NULL
    BEGIN
        IF NOT OBJECT_ID('v_Security_ErrorLog') IS NULL
        BEGIN
            INSERT INTO v_Security_ErrorLog(Message, Object, Operation, Login)
            SELECT @ErrorMessage, 't_Reporting_UserDashboard', 'INSERT', @UserName
        END
    END

   
    RETURN
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating trigger [dbo].[tru_Reporting_UserDashboard] on [dbo].[t_Reporting_UserDashboard]'
GO

CREATE TRIGGER [dbo].[tru_Reporting_UserDashboard] 
ON [dbo].[t_Reporting_UserDashboard]
AFTER UPDATE
AS

-- Prevent client from seing number of records affected
SET NOCOUNT ON

/* <PermissionCheck> */
    
/* </PermissionCheck> */

/* <CustomSection> */

/* </CustomSection> */

    GOTO TriggerContinue

TriggerError:
    -- Log error
    DECLARE @UserName AS NVARCHAR(128)
    SET @UserName = suser_sname()

    DECLARE @ErrorMessage AS NVARCHAR(512)
    SET @ErrorMessage = N'No permissions to UPDATE data [] for user [' + @UserName + ']'

    raiserror (@ErrorMessage, 14, 23)
    GOTO TriggerRollback

TriggerRollback:
    ROLLBACK TRANSACTION
    GOTO TriggerEnd

TriggerContinue:

    IF NOT OBJECT_ID('v_Security_DataLog') IS NULL
    BEGIN
        INSERT INTO v_Security_DataLog([Table], [Reference], [User], [Operation], [Time])
        SELECT 't_Reporting_UserDashboard', PrimKey, suser_sname(), 'U', getdate()
        FROM inserted
    END


TriggerEnd:

    IF NOT @ErrorMessage IS NULL
    BEGIN
        IF NOT OBJECT_ID('v_Security_ErrorLog') IS NULL
        BEGIN
            INSERT INTO v_Security_ErrorLog(Message, Object, Operation, Login)
            SELECT @ErrorMessage, 't_Reporting_UserDashboard', 'UPDATE', @UserName
        END
    END

   
    RETURN
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[v_Project_PersonAvailability]'
GO
EXEC sp_refreshview N'[dbo].[v_Project_PersonAvailability]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[vc_Project_PersonLookup]'
GO
EXEC sp_refreshview N'[dbo].[vc_Project_PersonLookup]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating trigger [dbo].[trd_Reporting_UserDashboard] on [dbo].[t_Reporting_UserDashboard]'
GO

CREATE TRIGGER [dbo].[trd_Reporting_UserDashboard] 
ON [dbo].[t_Reporting_UserDashboard]
AFTER DELETE
AS

-- Prevent client from seing number of records affected
SET NOCOUNT ON

/* <PermissionCheck> */
    
/* </PermissionCheck> */

/* <CustomSection> */

/* </CustomSection> */

    GOTO TriggerContinue

TriggerError:
    -- Log error
    DECLARE @UserName AS NVARCHAR(128)
    SET @UserName = suser_sname()

    DECLARE @ErrorMessage AS NVARCHAR(512)
    SET @ErrorMessage = N'No permissions to DELETE data [] for user [' + @UserName + ']'

    raiserror (@ErrorMessage, 14, 23)
    GOTO TriggerRollback

TriggerRollback:
    ROLLBACK TRANSACTION
    GOTO TriggerEnd

TriggerContinue:

    IF NOT OBJECT_ID('v_Security_DataLog') IS NULL
    BEGIN
        INSERT INTO v_Security_DataLog([Table], [Reference], [User], [Operation], [Time])
        SELECT 't_Reporting_UserDashboard', PrimKey, suser_sname(), 'D', getdate()
        FROM deleted
    END


TriggerEnd:

    IF NOT @ErrorMessage IS NULL
    BEGIN
        IF NOT OBJECT_ID('v_Security_ErrorLog') IS NULL
        BEGIN
            INSERT INTO v_Security_ErrorLog(Message, Object, Operation, Login)
            SELECT @ErrorMessage, 't_Reporting_UserDashboard', 'DELETE', @UserName
        END
    END

   
    RETURN
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[vc_DataSource_PersonAvailability]'
GO
EXEC sp_refreshview N'[dbo].[vc_DataSource_PersonAvailability]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Altering [dbo].[t_Project_PersonAvailabilityException]'
GO
ALTER TABLE [dbo].[t_Project_PersonAvailabilityException] ADD
[ApprovalDate] [datetime] NULL,
[ImportID] [nvarchar] (100) COLLATE Latin1_General_CI_AS NULL
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[v_Project_PersonAvailabilityException]'
GO
EXEC sp_refreshview N'[dbo].[v_Project_PersonAvailabilityException]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[v_Project_PersonValue]'
GO
EXEC sp_refreshview N'[dbo].[v_Project_PersonValue]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[v_Project_ProjectRevisionValue]'
GO
EXEC sp_refreshview N'[dbo].[v_Project_ProjectRevisionValue]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Altering [dbo].[vc_Reporting_DashboardWithoutLayout]'
GO


ALTER VIEW [dbo].[vc_Reporting_DashboardWithoutLayout]
AS
SELECT Title, [Description], ThumbnailImage, DataSourceRef, PrimKey, [TimeStamp]
FROM v_Reporting_Dashboard


GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating trigger [dbo].[tri_Project_ProjectStatus] on [dbo].[t_Project_ProjectStatus]'
GO

CREATE TRIGGER [dbo].[tri_Project_ProjectStatus] 
ON [dbo].[t_Project_ProjectStatus]
AFTER INSERT
AS

-- Prevent client from seing number of records affected
SET NOCOUNT ON

/* <PermissionCheck> */
    
/* </PermissionCheck> */

/* <CustomSection> */

/* </CustomSection> */

    GOTO TriggerContinue

TriggerError:
    -- Log error
    DECLARE @UserName AS NVARCHAR(128)
    SET @UserName = suser_sname()

    DECLARE @ErrorMessage AS NVARCHAR(512)
    SET @ErrorMessage = N'No permissions to INSERT data [] for user [' + @UserName + ']'

    raiserror (@ErrorMessage, 14, 23)
    GOTO TriggerRollback

TriggerRollback:
    ROLLBACK TRANSACTION
    GOTO TriggerEnd

TriggerContinue:

    IF NOT OBJECT_ID('v_Security_DataLog') IS NULL
    BEGIN
        INSERT INTO v_Security_DataLog([Table], [Reference], [User], [Operation], [Time])
        SELECT 't_Project_ProjectStatus', PrimKey, suser_sname(), 'I', getdate()
        FROM inserted
    END


TriggerEnd:

    IF NOT @ErrorMessage IS NULL
    BEGIN
        IF NOT OBJECT_ID('v_Security_ErrorLog') IS NULL
        BEGIN
            INSERT INTO v_Security_ErrorLog(Message, Object, Operation, Login)
            SELECT @ErrorMessage, 't_Project_ProjectStatus', 'INSERT', @UserName
        END
    END

   
    RETURN
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating trigger [dbo].[tru_Project_ProjectStatus] on [dbo].[t_Project_ProjectStatus]'
GO

CREATE TRIGGER [dbo].[tru_Project_ProjectStatus] 
ON [dbo].[t_Project_ProjectStatus]
AFTER UPDATE
AS

-- Prevent client from seing number of records affected
SET NOCOUNT ON

/* <PermissionCheck> */
    
/* </PermissionCheck> */

/* <CustomSection> */

/* </CustomSection> */

    GOTO TriggerContinue

TriggerError:
    -- Log error
    DECLARE @UserName AS NVARCHAR(128)
    SET @UserName = suser_sname()

    DECLARE @ErrorMessage AS NVARCHAR(512)
    SET @ErrorMessage = N'No permissions to UPDATE data [] for user [' + @UserName + ']'

    raiserror (@ErrorMessage, 14, 23)
    GOTO TriggerRollback

TriggerRollback:
    ROLLBACK TRANSACTION
    GOTO TriggerEnd

TriggerContinue:

    IF NOT OBJECT_ID('v_Security_DataLog') IS NULL
    BEGIN
        INSERT INTO v_Security_DataLog([Table], [Reference], [User], [Operation], [Time])
        SELECT 't_Project_ProjectStatus', PrimKey, suser_sname(), 'U', getdate()
        FROM inserted
    END


TriggerEnd:

    IF NOT @ErrorMessage IS NULL
    BEGIN
        IF NOT OBJECT_ID('v_Security_ErrorLog') IS NULL
        BEGIN
            INSERT INTO v_Security_ErrorLog(Message, Object, Operation, Login)
            SELECT @ErrorMessage, 't_Project_ProjectStatus', 'UPDATE', @UserName
        END
    END

   
    RETURN
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating trigger [dbo].[trd_Project_ProjectStatus] on [dbo].[t_Project_ProjectStatus]'
GO

CREATE TRIGGER [dbo].[trd_Project_ProjectStatus] 
ON [dbo].[t_Project_ProjectStatus]
AFTER DELETE
AS

-- Prevent client from seing number of records affected
SET NOCOUNT ON

/* <PermissionCheck> */
    
/* </PermissionCheck> */

/* <CustomSection> */

/* </CustomSection> */

    GOTO TriggerContinue

TriggerError:
    -- Log error
    DECLARE @UserName AS NVARCHAR(128)
    SET @UserName = suser_sname()

    DECLARE @ErrorMessage AS NVARCHAR(512)
    SET @ErrorMessage = N'No permissions to DELETE data [] for user [' + @UserName + ']'

    raiserror (@ErrorMessage, 14, 23)
    GOTO TriggerRollback

TriggerRollback:
    ROLLBACK TRANSACTION
    GOTO TriggerEnd

TriggerContinue:

    IF NOT OBJECT_ID('v_Security_DataLog') IS NULL
    BEGIN
        INSERT INTO v_Security_DataLog([Table], [Reference], [User], [Operation], [Time])
        SELECT 't_Project_ProjectStatus', PrimKey, suser_sname(), 'D', getdate()
        FROM deleted
    END


TriggerEnd:

    IF NOT @ErrorMessage IS NULL
    BEGIN
        IF NOT OBJECT_ID('v_Security_ErrorLog') IS NULL
        BEGIN
            INSERT INTO v_Security_ErrorLog(Message, Object, Operation, Login)
            SELECT @ErrorMessage, 't_Project_ProjectStatus', 'DELETE', @UserName
        END
    END

   
    RETURN
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating trigger [dbo].[tri_Project_ProjectType] on [dbo].[t_Project_ProjectType]'
GO

CREATE TRIGGER [dbo].[tri_Project_ProjectType] 
ON [dbo].[t_Project_ProjectType]
AFTER INSERT
AS

-- Prevent client from seing number of records affected
SET NOCOUNT ON

/* <PermissionCheck> */
    
/* </PermissionCheck> */

/* <CustomSection> */

/* </CustomSection> */

    GOTO TriggerContinue

TriggerError:
    -- Log error
    DECLARE @UserName AS NVARCHAR(128)
    SET @UserName = suser_sname()

    DECLARE @ErrorMessage AS NVARCHAR(512)
    SET @ErrorMessage = N'No permissions to INSERT data [] for user [' + @UserName + ']'

    raiserror (@ErrorMessage, 14, 23)
    GOTO TriggerRollback

TriggerRollback:
    ROLLBACK TRANSACTION
    GOTO TriggerEnd

TriggerContinue:

    IF NOT OBJECT_ID('v_Security_DataLog') IS NULL
    BEGIN
        INSERT INTO v_Security_DataLog([Table], [Reference], [User], [Operation], [Time])
        SELECT 't_Project_ProjectType', PrimKey, suser_sname(), 'I', getdate()
        FROM inserted
    END


TriggerEnd:

    IF NOT @ErrorMessage IS NULL
    BEGIN
        IF NOT OBJECT_ID('v_Security_ErrorLog') IS NULL
        BEGIN
            INSERT INTO v_Security_ErrorLog(Message, Object, Operation, Login)
            SELECT @ErrorMessage, 't_Project_ProjectType', 'INSERT', @UserName
        END
    END

   
    RETURN
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating trigger [dbo].[tru_Project_ProjectType] on [dbo].[t_Project_ProjectType]'
GO

CREATE TRIGGER [dbo].[tru_Project_ProjectType] 
ON [dbo].[t_Project_ProjectType]
AFTER UPDATE
AS

-- Prevent client from seing number of records affected
SET NOCOUNT ON

/* <PermissionCheck> */
    
/* </PermissionCheck> */

/* <CustomSection> */

/* </CustomSection> */

    GOTO TriggerContinue

TriggerError:
    -- Log error
    DECLARE @UserName AS NVARCHAR(128)
    SET @UserName = suser_sname()

    DECLARE @ErrorMessage AS NVARCHAR(512)
    SET @ErrorMessage = N'No permissions to UPDATE data [] for user [' + @UserName + ']'

    raiserror (@ErrorMessage, 14, 23)
    GOTO TriggerRollback

TriggerRollback:
    ROLLBACK TRANSACTION
    GOTO TriggerEnd

TriggerContinue:

    IF NOT OBJECT_ID('v_Security_DataLog') IS NULL
    BEGIN
        INSERT INTO v_Security_DataLog([Table], [Reference], [User], [Operation], [Time])
        SELECT 't_Project_ProjectType', PrimKey, suser_sname(), 'U', getdate()
        FROM inserted
    END


TriggerEnd:

    IF NOT @ErrorMessage IS NULL
    BEGIN
        IF NOT OBJECT_ID('v_Security_ErrorLog') IS NULL
        BEGIN
            INSERT INTO v_Security_ErrorLog(Message, Object, Operation, Login)
            SELECT @ErrorMessage, 't_Project_ProjectType', 'UPDATE', @UserName
        END
    END

   
    RETURN
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[v_MPP_PositionTypeValue]'
GO
EXEC sp_refreshview N'[dbo].[v_MPP_PositionTypeValue]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Creating trigger [dbo].[trd_Project_ProjectType] on [dbo].[t_Project_ProjectType]'
GO

CREATE TRIGGER [dbo].[trd_Project_ProjectType] 
ON [dbo].[t_Project_ProjectType]
AFTER DELETE
AS

-- Prevent client from seing number of records affected
SET NOCOUNT ON

/* <PermissionCheck> */
    
/* </PermissionCheck> */

/* <CustomSection> */

/* </CustomSection> */

    GOTO TriggerContinue

TriggerError:
    -- Log error
    DECLARE @UserName AS NVARCHAR(128)
    SET @UserName = suser_sname()

    DECLARE @ErrorMessage AS NVARCHAR(512)
    SET @ErrorMessage = N'No permissions to DELETE data [] for user [' + @UserName + ']'

    raiserror (@ErrorMessage, 14, 23)
    GOTO TriggerRollback

TriggerRollback:
    ROLLBACK TRANSACTION
    GOTO TriggerEnd

TriggerContinue:

    IF NOT OBJECT_ID('v_Security_DataLog') IS NULL
    BEGIN
        INSERT INTO v_Security_DataLog([Table], [Reference], [User], [Operation], [Time])
        SELECT 't_Project_ProjectType', PrimKey, suser_sname(), 'D', getdate()
        FROM deleted
    END


TriggerEnd:

    IF NOT @ErrorMessage IS NULL
    BEGIN
        IF NOT OBJECT_ID('v_Security_ErrorLog') IS NULL
        BEGIN
            INSERT INTO v_Security_ErrorLog(Message, Object, Operation, Login)
            SELECT @ErrorMessage, 't_Project_ProjectType', 'DELETE', @UserName
        END
    END

   
    RETURN
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[v_Security_OrganizationUnitRights]'
GO
EXEC sp_refreshview N'[dbo].[v_Security_OrganizationUnitRights]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[v_Project_SelectedProject]'
GO
EXEC sp_refreshview N'[dbo].[v_Project_SelectedProject]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Refreshing [dbo].[v_Security_ProjectRights]'
GO
EXEC sp_refreshview N'[dbo].[v_Security_ProjectRights]'
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Adding foreign keys to [dbo].[t_Project_OrganizationUnit]'
GO
ALTER TABLE [dbo].[t_Project_OrganizationUnit] ADD CONSTRAINT [r_Project_Person_HandlesAbsenceInOrganizationUnits] FOREIGN KEY ([AbsenceHandlerRef]) REFERENCES [dbo].[t_Project_Person] ([PrimKey])
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Adding foreign keys to [dbo].[t_Project_Project]'
GO
ALTER TABLE [dbo].[t_Project_Project] ADD CONSTRAINT [r_Project_ProjectType_Projects] FOREIGN KEY ([ProjectTypeRef]) REFERENCES [dbo].[t_Project_ProjectType] ([PrimKey])
ALTER TABLE [dbo].[t_Project_Project] ADD CONSTRAINT [r_Project_ProjectStatus_Projects] FOREIGN KEY ([ProjectStatusRef]) REFERENCES [dbo].[t_Project_ProjectStatus] ([PrimKey])
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Adding foreign keys to [dbo].[t_Project_ProjectStatus]'
GO
ALTER TABLE [dbo].[t_Project_ProjectStatus] ADD CONSTRAINT [r_Project_Revision_ProjectStatus] FOREIGN KEY ([RevisionRef]) REFERENCES [dbo].[t_Project_Revision] ([PrimKey])
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Adding foreign keys to [dbo].[t_Project_ProjectType]'
GO
ALTER TABLE [dbo].[t_Project_ProjectType] ADD CONSTRAINT [r_Project_Revision_ProjectTypes] FOREIGN KEY ([RevisionRef]) REFERENCES [dbo].[t_Project_Revision] ([PrimKey])
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Adding foreign keys to [dbo].[t_Reporting_DashboardTag]'
GO
ALTER TABLE [dbo].[t_Reporting_DashboardTag] ADD CONSTRAINT [r_Reporting_Dashboard_Tags] FOREIGN KEY ([DashboardRef]) REFERENCES [dbo].[t_Reporting_Dashboard] ([PrimKey]) ON DELETE CASCADE
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Adding foreign keys to [dbo].[t_Reporting_UserDashboard]'
GO
ALTER TABLE [dbo].[t_Reporting_UserDashboard] ADD CONSTRAINT [r_Reporting_Report_UserDashboards] FOREIGN KEY ([DashboardRef]) REFERENCES [dbo].[t_Reporting_Dashboard] ([PrimKey]) ON DELETE CASCADE
ALTER TABLE [dbo].[t_Reporting_UserDashboard] ADD CONSTRAINT [r_Security_User_UserDashboards] FOREIGN KEY ([UserRef]) REFERENCES [dbo].[t_Security_User] ([PrimKey])
GO
IF @@ERROR<>0 AND @@TRANCOUNT>0 ROLLBACK TRANSACTION
GO

GO
PRINT N'Altering permissions on [dbo].[v_Project_ProjectStatus]'
GO
GRANT SELECT ON  [dbo].[v_Project_ProjectStatus] TO [public]
GRANT INSERT ON  [dbo].[v_Project_ProjectStatus] TO [public]
GRANT DELETE ON  [dbo].[v_Project_ProjectStatus] TO [public]
GRANT UPDATE ON  [dbo].[v_Project_ProjectStatus] TO [public]
GO
PRINT N'Altering permissions on [dbo].[v_Project_ProjectType]'
GO
GRANT SELECT ON  [dbo].[v_Project_ProjectType] TO [public]
GRANT INSERT ON  [dbo].[v_Project_ProjectType] TO [public]
GRANT DELETE ON  [dbo].[v_Project_ProjectType] TO [public]
GRANT UPDATE ON  [dbo].[v_Project_ProjectType] TO [public]
GO
PRINT N'Altering permissions on [dbo].[v_Reporting_DashboardTag]'
GO
GRANT SELECT ON  [dbo].[v_Reporting_DashboardTag] TO [public]
GRANT INSERT ON  [dbo].[v_Reporting_DashboardTag] TO [public]
GRANT DELETE ON  [dbo].[v_Reporting_DashboardTag] TO [public]
GRANT UPDATE ON  [dbo].[v_Reporting_DashboardTag] TO [public]
GO
PRINT N'Altering permissions on [dbo].[v_Reporting_UserDashboard]'
GO
GRANT SELECT ON  [dbo].[v_Reporting_UserDashboard] TO [public]
GRANT INSERT ON  [dbo].[v_Reporting_UserDashboard] TO [public]
GRANT DELETE ON  [dbo].[v_Reporting_UserDashboard] TO [public]
GRANT UPDATE ON  [dbo].[v_Reporting_UserDashboard] TO [public]
GO

COMMIT TRANSACTION

