/* To prevent any potential data loss issues, you should review this script in detail before running it outside the context of the database designer.*/
BEGIN TRANSACTION
SET QUOTED_IDENTIFIER ON
SET ARITHABORT ON
SET NUMERIC_ROUNDABORT OFF
SET CONCAT_NULL_YIELDS_NULL ON
SET ANSI_NULLS ON
SET ANSI_PADDING ON
SET ANSI_WARNINGS ON
COMMIT
BEGIN TRANSACTION
GO
ALTER TABLE dbo.t_Project_Revision SET (LOCK_ESCALATION = TABLE)
GO
COMMIT
BEGIN TRANSACTION
GO
ALTER TABLE dbo.t_MPP_ProjectPositionField
	DROP CONSTRAINT r_Project_ProjectRevision_ProjectPositionFields
GO
ALTER TABLE dbo.t_Project_ProjectRevision SET (LOCK_ESCALATION = TABLE)
GO
COMMIT
BEGIN TRANSACTION
GO
ALTER TABLE dbo.t_MPP_ProjectPositionField
	DROP CONSTRAINT r_Project_Field_ProjectPositionFields
GO
ALTER TABLE dbo.t_Project_Field SET (LOCK_ESCALATION = TABLE)
GO
COMMIT
BEGIN TRANSACTION
GO
ALTER TABLE dbo.t_MPP_ProjectPositionField
	DROP CONSTRAINT DF_t_MPP_ProjectField_PrimKey
GO
CREATE TABLE dbo.Tmp_t_MPP_ProjectPositionField
	(
	RevisionRef uniqueidentifier NULL,
	ProjectRevisionRef uniqueidentifier NOT NULL,
	FieldRef uniqueidentifier NOT NULL,
	Required bit NOT NULL,
	PrimKey uniqueidentifier NOT NULL,
	TimeStamp timestamp NOT NULL
	)  ON [PRIMARY]
GO
ALTER TABLE dbo.Tmp_t_MPP_ProjectPositionField SET (LOCK_ESCALATION = TABLE)
GO
ALTER TABLE dbo.Tmp_t_MPP_ProjectPositionField ADD CONSTRAINT
	DF_t_MPP_ProjectField_PrimKey DEFAULT (newsequentialid()) FOR PrimKey
GO
IF EXISTS(SELECT * FROM dbo.t_MPP_ProjectPositionField)
	 EXEC('INSERT INTO dbo.Tmp_t_MPP_ProjectPositionField (ProjectRevisionRef, FieldRef, Required, PrimKey)
		SELECT ProjectRevisionRef, FieldRef, Required, PrimKey FROM dbo.t_MPP_ProjectPositionField WITH (HOLDLOCK TABLOCKX)')
GO
DROP TABLE dbo.t_MPP_ProjectPositionField
GO
EXECUTE sp_rename N'dbo.Tmp_t_MPP_ProjectPositionField', N't_MPP_ProjectPositionField', 'OBJECT' 
GO
ALTER TABLE dbo.t_MPP_ProjectPositionField ADD CONSTRAINT
	PK_t_MPP_ProjectField PRIMARY KEY CLUSTERED 
	(
	PrimKey
	) WITH( STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]

GO
ALTER TABLE dbo.t_MPP_ProjectPositionField ADD CONSTRAINT
	r_Project_Field_ProjectPositionFields FOREIGN KEY
	(
	FieldRef
	) REFERENCES dbo.t_Project_Field
	(
	PrimKey
	) ON UPDATE  NO ACTION 
	 ON DELETE  CASCADE 
	
GO
ALTER TABLE dbo.t_MPP_ProjectPositionField ADD CONSTRAINT
	r_Project_ProjectRevision_ProjectPositionFields FOREIGN KEY
	(
	ProjectRevisionRef
	) REFERENCES dbo.t_Project_ProjectRevision
	(
	PrimKey
	) ON UPDATE  NO ACTION 
	 ON DELETE  CASCADE 
	
GO
ALTER TABLE dbo.t_MPP_ProjectPositionField ADD CONSTRAINT
	r_Project_Revision_ProjectPositionFields FOREIGN KEY
	(
	RevisionRef
	) REFERENCES dbo.t_Project_Revision
	(
	PrimKey
	) ON UPDATE  NO ACTION 
	 ON DELETE  NO ACTION 
	
GO
CREATE TRIGGER [tri_MPP_ProjectPositionField] 
ON dbo.t_MPP_ProjectPositionField
AFTER INSERT
AS

-- Prevent client from seing number of records affected
SET NOCOUNT ON

/* <PermissionCheck> */
    
/* </PermissionCheck> */

/* <CustomSection> */

/* </CustomSection> */

    GOTO TriggerContinue

TriggerError:
    -- Log error
    DECLARE @UserName AS NVARCHAR(128)
    SET @UserName = suser_sname()

    DECLARE @ErrorMessage AS NVARCHAR(512)
    SET @ErrorMessage = N'No permissions to INSERT data [] for user [' + @UserName + ']'

    raiserror (@ErrorMessage, 14, 23)
    GOTO TriggerRollback

TriggerRollback:
    ROLLBACK TRANSACTION
    GOTO TriggerEnd

TriggerContinue:

    IF NOT OBJECT_ID('v_Security_DataLog') IS NULL
    BEGIN
        INSERT INTO v_Security_DataLog([Table], [Reference], [User], [Operation], [Time])
        SELECT 't_MPP_ProjectPositionField', PrimKey, suser_sname(), 'I', getdate()
        FROM inserted
    END


TriggerEnd:

    IF NOT @ErrorMessage IS NULL
    BEGIN
        IF NOT OBJECT_ID('v_Security_ErrorLog') IS NULL
        BEGIN
            INSERT INTO v_Security_ErrorLog(Message, Object, Operation, Login)
            SELECT @ErrorMessage, 't_MPP_ProjectPositionField', 'INSERT', @UserName
        END
    END

   
    RETURN
GO
CREATE TRIGGER [tru_MPP_ProjectPositionField] 
ON dbo.t_MPP_ProjectPositionField
AFTER UPDATE
AS

-- Prevent client from seing number of records affected
SET NOCOUNT ON

/* <PermissionCheck> */
    
/* </PermissionCheck> */

/* <CustomSection> */

/* </CustomSection> */

    GOTO TriggerContinue

TriggerError:
    -- Log error
    DECLARE @UserName AS NVARCHAR(128)
    SET @UserName = suser_sname()

    DECLARE @ErrorMessage AS NVARCHAR(512)
    SET @ErrorMessage = N'No permissions to UPDATE data [] for user [' + @UserName + ']'

    raiserror (@ErrorMessage, 14, 23)
    GOTO TriggerRollback

TriggerRollback:
    ROLLBACK TRANSACTION
    GOTO TriggerEnd

TriggerContinue:

    IF NOT OBJECT_ID('v_Security_DataLog') IS NULL
    BEGIN
        INSERT INTO v_Security_DataLog([Table], [Reference], [User], [Operation], [Time])
        SELECT 't_MPP_ProjectPositionField', PrimKey, suser_sname(), 'U', getdate()
        FROM inserted
    END


TriggerEnd:

    IF NOT @ErrorMessage IS NULL
    BEGIN
        IF NOT OBJECT_ID('v_Security_ErrorLog') IS NULL
        BEGIN
            INSERT INTO v_Security_ErrorLog(Message, Object, Operation, Login)
            SELECT @ErrorMessage, 't_MPP_ProjectPositionField', 'UPDATE', @UserName
        END
    END

   
    RETURN
GO
CREATE TRIGGER [trd_MPP_ProjectPositionField] 
ON dbo.t_MPP_ProjectPositionField
AFTER DELETE
AS

-- Prevent client from seing number of records affected
SET NOCOUNT ON

/* <PermissionCheck> */
    
/* </PermissionCheck> */

/* <CustomSection> */

/* </CustomSection> */

    GOTO TriggerContinue

TriggerError:
    -- Log error
    DECLARE @UserName AS NVARCHAR(128)
    SET @UserName = suser_sname()

    DECLARE @ErrorMessage AS NVARCHAR(512)
    SET @ErrorMessage = N'No permissions to DELETE data [] for user [' + @UserName + ']'

    raiserror (@ErrorMessage, 14, 23)
    GOTO TriggerRollback

TriggerRollback:
    ROLLBACK TRANSACTION
    GOTO TriggerEnd

TriggerContinue:

    IF NOT OBJECT_ID('v_Security_DataLog') IS NULL
    BEGIN
        INSERT INTO v_Security_DataLog([Table], [Reference], [User], [Operation], [Time])
        SELECT 't_MPP_ProjectPositionField', PrimKey, suser_sname(), 'D', getdate()
        FROM deleted
    END


TriggerEnd:

    IF NOT @ErrorMessage IS NULL
    BEGIN
        IF NOT OBJECT_ID('v_Security_ErrorLog') IS NULL
        BEGIN
            INSERT INTO v_Security_ErrorLog(Message, Object, Operation, Login)
            SELECT @ErrorMessage, 't_MPP_ProjectPositionField', 'DELETE', @UserName
        END
    END

   
    RETURN
GO
COMMIT


/* SET RevisionRef and remove duplicates */
begin transaction
UPDATE F
SET RevisionRef = PR.RevisionRef
FROM t_MPP_ProjectPositionField F
	JOIN t_Project_ProjectRevision PR ON F.ProjectRevisionRef = PR.PrimKey

DELETE F
FROM t_MPP_ProjectPositionField F
WHERE EXISTS (select * from t_MPP_ProjectPositionField F2 WHERE F2.RevisionRef = F.RevisionRef AND F2.FieldRef = F.FieldRef AND F2.PrimKey > F.PrimKey)
commit
GO



/* Add RevisionRef - FieldRef Contraint */
BEGIN TRANSACTION
GO
CREATE UNIQUE NONCLUSTERED INDEX IX_t_MPP_ProjectPositionField ON dbo.t_MPP_ProjectPositionField
	(
	RevisionRef,
	FieldRef
	) WITH( STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
GO
ALTER TABLE dbo.t_MPP_ProjectPositionField SET (LOCK_ESCALATION = TABLE)
GO
COMMIT


/* Remove ProjectRevisionRef and relation*/
BEGIN TRANSACTION
GO
ALTER TABLE dbo.t_MPP_ProjectPositionField
	DROP CONSTRAINT r_Project_ProjectRevision_ProjectPositionFields
GO
ALTER TABLE dbo.t_Project_ProjectRevision SET (LOCK_ESCALATION = TABLE)
GO
COMMIT
BEGIN TRANSACTION
GO
ALTER TABLE dbo.t_MPP_ProjectPositionField
	DROP COLUMN ProjectRevisionRef
GO
ALTER TABLE dbo.t_MPP_ProjectPositionField SET (LOCK_ESCALATION = TABLE)
GO
COMMIT


EXEC sp_refreshview 'v_MPP_ProjectPositionField'
GO
